
/*
BMP180 Pressure and temperature sensor
Reads the BMP180 sensor via wi2c
and converts the voltage value to g-value. 
*/

#include <SerialPort.h>             // (http://arduino.cc/forum/index.php/topic,85207.0.html)
#include <Statistic.h>              // Statistics library: http://arduino.cc/playground/Main/Statistics
#include <i2c.h>

#define bmp_adress 0xEE             // The even|write|smaller adress

// USE_NEW_SERIAL;                  // Does not interfere with interrupts

Statistic Stat;
int wait = 1;             // 1/2 wait time between measurements
double g_value, sigma;
float ac1, ac2, ac3, ac4, ac5, ac6, b1, b2, mb, mc, md;
byte oss = 3; //ultra high resolution sampling
float last_temp, last_press, temp_std, press_std;

void setup() {
  Serial.begin (115200);                 // Set up the Serial Connection.
  Serial.println("Reading BMP180");      // Greet user.
}

void loop() {
  long int i;
  int values[500];

Stat.clear(); 
i=0;
while(i<1000)  {
  i++;
}
  g_value = Stat.average();
  sigma = Stat.average() + Stat.pop_stdev() - g_value;
  Serial.println("Something");  
  delay(wait);
}

//byte1 = MSB, byte2 = LSB
static short ToShort(short byte1, short byte2){  
  return (byte1 << 8) + byte2;
}

static long ToLong(long byte1, long byte2, long byte3, short byte4){
  return (byte1 << 24L) + (byte2 << 16L) +(byte3 << 8L) + byte4;
}

//byte1 = MSB, byte2 = LSB
static unsigned short ToUnShort(unsigned short byte1, unsigned short byte2){
  return (byte1 << 8) + byte2;
}

void read_calibration_float(){
  // Read all the BMP180 calibration variables via wi2c
  i2c_start();
  while(!i2c_write(bmp_adress)){
  }
  while(i2c_write(0xAA)==0){
  }
  i2c_start();
  while(!i2c_write(bmp_adress+1)){
  }
  ac1=(float) ToShort(i2c_read(1), i2c_read(1));
  ac2=(float) ToShort(i2c_read(1), i2c_read(1));
  ac3=(float) ToShort(i2c_read(1), i2c_read(1));
  ac4=(float) ToUnShort(i2c_read(1), i2c_read(1));
  ac5=(float) ToUnShort(i2c_read(1), i2c_read(1));
  ac6=(float) ToUnShort(i2c_read(1), i2c_read(1));
  b1=(float) ToShort(i2c_read(1), i2c_read(1));
  b2=(float) ToShort(i2c_read(1), i2c_read(1));
  mb=(float) ToShort(i2c_read(1), i2c_read(1));
  mc=(float) ToShort(i2c_read(1), i2c_read(1));
  md=(float) ToShort(i2c_read(1), i2c_read(0));

  i2c_stop();
}

long read_temperature_bmp(){
  // Tell BMP180 that you want to measure the temperature 
  i2c_start();
  while(!i2c_write(bmp_adress)){
  }
  while(!i2c_write(0xF4)){
  }
  while(!i2c_write(0x2E)){ //write 0x2E into register 0xF4
  }
  i2c_stop();

  NewDelay(5); //Datasheet: Delay >= 4.5ms required 

  i2c_start();
  while(!i2c_write(bmp_adress)){
  }
  while(!i2c_write(0xF6)){ // Tell BMP we want to read from F6
  }

  i2c_start(); // wi2c restart
  while(!i2c_write(bmp_adress+0x1)){
  }

  long ut=ToLong(0,0,i2c_read(1),i2c_read(0));
  i2c_stop();

  return ut;
}

float read_pressure_float(){
  i2c_start();
  while(!i2c_write(bmp_adress)){
  }
  while(!i2c_write(0xF4)){
  }
  while(!i2c_write(0x34 + (oss << 6))){
  }
  i2c_stop();

  NewDelay(26); //Check data sheet for shortest possible time (depends on oss settings)

  i2c_start();
  while(!i2c_write(bmp_adress)){
  }
  while(!i2c_write(0xF6)){
  }

  i2c_start();
  while(!i2c_write(bmp_adress+0x01)){
  }

  long up=ToLong(0, i2c_read(1),i2c_read(1),i2c_read(0));
  up/=pow(2,8-oss);
  i2c_stop();
  return up;
}

void measure_bmp180_float(double *t, double *p){
  double ut, x1, x2, x3, b4, b5, b6, b3, b7;
  Statistic up;

  ut=(double) read_temperature_bmp();

  for(int i=0;i<10;i++)
    up.add(read_pressure_float());

  // Calculations are obtained from datasheet. We use floats instead of int/long 
  // in order to reduce rounding errors. (Power consumption or speed is not crucial)
  x1 = (ut - ac6) * ac5 / 32768.0;
  x2 = mc * 2048.0 / (x1 + md) ;
  b5 = x1 + x2;
  *t = (b5 + 8.0) / 160.0;

  b6 = b5 - 4000.0;
  x1 = (b2 * (b6 * b6 / 4096.0 )) / 2048.0;
  x2 = ac2 * b6 / 2048.0;
  x3 = x1 + x2;
  b3 = ((ac1* 4L + x3) * pow(2,oss) + 2.0) / 4.0;
  x1 = ac3 * b6 / 8192.0;
  x2 = (b1 * (b6 * b6 / 4096.0 )) / 65536.0;
  x3 = ((x1 + x2) + 2.0) / 4.0;
  b4 = ac4 * (x3 + 32768.0) / 32768.0;
  b7 = (up.average() - b3) * (50000.0 / pow(2,oss));
  *p = b7 / b4 * 2.0;

  x1 = (*p / 256.0 ) * (*p / 256.0 );
  x1 = (x1 * 3038.0 ) / 65536.0;
  x2 = (-7357.0 * *p) / 65536.0;
  *p = *p + (x1 + x2 + 3791.0) / 16.0;
}

void NewDelay(unsigned int time){
  // Checks for Serial communication and updates LCD until time is up
  long time2=millis();
  while (millis()-time2 <= time){
//    update_communication();
  }
}

